==> buckshot-0.3.0.jar: buckshot/backend_redis.clj <==
(in-ns 'buckshot.backend)
(require '[taoensso.carmine :as redis])

(def pool (redis/make-conn-pool :max-active -1
                                :max-idle -1))

(defmacro with-conn [& body]
  `(redis/with-conn pool ~'spec
     ~@body))

(defn- txn-success? [results]
  (and (seq results)
       (every? #{1} results)))

(defrecord RedisBackend [spec listener]
  IBackend
  (get-colls [_]
    (->> (with-conn
           (redis/keys "*"))
         (map keyword)))
  (get-all [_ coll]
    (with-conn
      (redis/zrange (name coll) 0 -1)))
  (get-min [_ coll max]
    (-> (with-conn
          (redis/zrangebyscore (name coll) "-inf" max "LIMIT" 0 1))
        first))
  (has-score? [_ coll score]
    (-> (with-conn
          (redis/zrangebyscore (name coll) score score "LIMIT" 0 1))
        seq))
  (has-item? [_ coll item]
    (with-conn
      (redis/zscore (name coll) item)))
  (atomically [_ f]
    (-> (with-conn
          (redis/atomically [] (f)))
        txn-success?))
  (add! [_ coll score item]
    (redis/zadd (name coll) score item))
  (remove! [_ coll item]
    (redis/zrem (name coll) item))
  (delete! [_ coll]
    (redis/del (name coll)))
  (publish! [_ channel message]
    (with-conn
      (redis/publish (name channel) message)))
  (subscribe! [_ channel f]
    (swap! (:state listener) assoc (name channel) (fn [[type channel message]]
                                                    (when (= type "message")
                                                      (f message))))
    (redis/with-open-listener listener
      (redis/subscribe (name channel)))))

(defn make-redis [params]
  (let [spec (merge (redis/make-conn-spec) params)
        listener (redis/with-new-pubsub-listener spec {})]
    (RedisBackend. spec listener)))

==> buckshot-0.4.0.jar: buckshot/backend_redis.clj <==
(in-ns 'buckshot.backend)
(require '[taoensso.carmine :as redis])

(def pool (redis/make-conn-pool :max-active -1
                                :max-idle -1))

(defmacro with-conn [& body]
  `(redis/with-conn pool ~'spec
     ~@body))

(defn- txn-success? [results]
  (and (seq results)
       (every? #{1} results)))

(defrecord RedisBackend [spec listener]
  IBackend
  (colls [_]
    (->> (with-conn
           (redis/keys "*"))
         (map keyword)))
  (atomically [_ f]
    (-> (with-conn
          (redis/atomically [] (f)))
        txn-success?))
  (del! [_ coll]
    (redis/del (name coll)))
  (hall [_ hash]
    (with-conn
      (redis/hvals (name hash))))
  (hadd! [_ hash key value]
    (redis/hsetnx (name hash) key value))
  (hrem! [_ hash key]
    (redis/hdel (name hash) key))
  (zall [_ set]
    (with-conn
      (redis/zrange (name set) 0 -1)))
  (zexists? [_ set value]
    (with-conn
      (redis/zscore (name set) value)))
  (zmin [_ set max-key]
    (-> (with-conn
          (redis/zrangebyscore (name set) "-inf" max-key "LIMIT" 0 1))
        first))
  (zadd! [_ set key value]
    (redis/zadd (name set) key value))
  (zrem! [_ set value]
    (redis/zrem (name set) value))
  (publish! [_ channel message]
    (with-conn
      (redis/publish (name channel) message)))
  (subscribe! [_ channel f]
    (swap! (:state listener) assoc (name channel) (fn [[type ch msg]]
                                                    (when (= type "message")
                                                      (f msg))))
    (redis/with-open-listener listener
      (redis/subscribe (name channel)))))

(defn make-redis [params]
  (let [spec (merge (redis/make-conn-spec) params)
        listener (redis/with-new-pubsub-listener spec {})]
    (RedisBackend. spec listener)))
